package top.jerryyan.RN.A.VersionUpgrade;

import android.app.Activity;
import android.app.AlertDialog;
import android.app.ProgressDialog;
import android.content.DialogInterface;
import android.content.Intent;
import android.content.pm.PackageInfo;
import android.content.pm.PackageManager;
import android.net.Uri;
import android.os.Build;
import android.os.Environment;
import android.os.Handler;
import android.os.Message;
import android.provider.Settings;
import android.util.Log;

import androidx.core.content.FileProvider;

import com.facebook.react.bridge.ReactApplicationContext;
import com.facebook.react.bridge.ReactContextBaseJavaModule;
import com.facebook.react.bridge.ReactMethod;

import org.json.JSONException;
import org.json.JSONObject;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStream;
import java.util.HashMap;
import java.util.Map;

import okhttp3.Call;
import okhttp3.Callback;
import okhttp3.OkHttpClient;
import okhttp3.Request;
import okhttp3.Response;

public class UpgradeModule extends ReactContextBaseJavaModule {
    private final ReactApplicationContext reactContext;
    private String DOWNLOAD_URL = "http://luntan.qgmzbxs.com/source/tsgzCircles.apk";
    private String HOST = "http://www.tsgzvore.com/";
    private String TITLE = "更新";
    private String CONTENT = "大家好，我是勤劳的催更新菌\n点击更新，让我们一起成为一天更新八次的暴躁老哥吧";

    public UpgradeModule(ReactApplicationContext reactContext) {
        super(reactContext);
        this.reactContext = reactContext;
    }

    @Override
    public String getName() {
        return "VersionUpgrade";
    }

    @Override
    public Map<String, Object> getConstants() {
        final Map<String, Object> constants = new HashMap<>();
        return constants;
    }

    boolean _hasPermissionToInstall() {
        Activity activity = reactContext.getCurrentActivity();
        return activity != null && (Build.VERSION.SDK_INT < Build.VERSION_CODES.O || activity.getPackageManager().canRequestPackageInstalls());
    }

    void _requestInstallPermission() {
        Activity activity = reactContext.getCurrentActivity();
        if (activity == null) return;
        if (Build.VERSION.SDK_INT >= Build.VERSION_CODES.O) {
            Uri packageURI = Uri.parse("package:" + activity.getPackageName());
            Intent intent = new Intent(Settings.ACTION_MANAGE_UNKNOWN_APP_SOURCES, packageURI);
            (activity).startActivityForResult(intent, 2);
        }
    }

    void _alertDialog() {
        final UpgradeModule that = this;
        AlertDialog dialog = new AlertDialog.Builder(reactContext.getCurrentActivity())
                .setTitle(TITLE)
                .setMessage(CONTENT)
                .setNeutralButton("取消",
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                dialog.dismiss();
                            }
                        }
                )
                .setNegativeButton("更新",
                        new DialogInterface.OnClickListener() {
                            @Override
                            public void onClick(DialogInterface dialog, int which) {
                                that._startDownload();
                            }
                        }
                )
                .show();
    }

    void installPackage(String fileUri){
        Intent install = new Intent(Intent.ACTION_VIEW);
        Uri fileUri1 = Uri.parse(fileUri);
        File file = new File(fileUri1.getPath());
        if (Build.VERSION.SDK_INT >= 24) {
            Uri apkUri = FileProvider.getUriForFile(reactContext, reactContext.getPackageName().concat(".fileprovider"), file);
            install.addFlags(Intent.FLAG_GRANT_READ_URI_PERMISSION);
            install.setDataAndType(apkUri, "application/vnd.android.package-archive");
        } else {
            install.setDataAndType(Uri.parse(fileUri), "application/vnd.android.package-archive");
        }
        install.addFlags(Intent.FLAG_ACTIVITY_NEW_TASK);
        reactContext.startActivity(install);
    }

    private void _startDownload() {
        OkHttpClient okHttpClient = new OkHttpClient();
        final Request request = new Request.Builder()
                .url(DOWNLOAD_URL)
                .method("GET", null)
                .build();
        final Call call = okHttpClient.newCall(request);
        final Handler handler = new DoInstallHandler(this);
        if(!this._hasPermissionToInstall()) this._requestInstallPermission();
        call.enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                Log.e("PACKAGE DOWNLOAD", call.toString(), e);
            }

            @Override
            public void onResponse(Call call, Response response) {
                if (response.isSuccessful()) {
                    if (response.body() == null) return;
                    FileOutputStream fileOutputStream = null;
                    InputStream stream = null;
                    try {
                        stream = response.body().byteStream();
                        byte[] buffer = new byte[2048];
                        long total = response.body().contentLength();
                        int len;
                        long current = 0;
                        File file = new File(reactContext.getExternalFilesDir(Environment.DIRECTORY_DOWNLOADS).getAbsolutePath() ,"update.apk");
                        fileOutputStream = new FileOutputStream(file, false);
                        while ((len = stream.read(buffer)) != -1) {
                            fileOutputStream.write(buffer, 0, len);
                            current += len;
                            ProgressNotificationUtil.showNotificationProgressApkDown(reactContext, (int)((100.0f*current)/total));
                        }
                        fileOutputStream.flush();
                        final Message message = new Message();
                        message.what = 0;
                        message.obj = file.toURI().toString();
                        handler.sendMessage(message);
                    } catch (IOException e) {
                        Log.e("REQUEST FAILED", "AAA", e);
                    } finally {
                        try {
                            if (fileOutputStream != null) fileOutputStream.close();
                            if (stream != null) stream.close();
                        } catch (IOException e) {
                            e.printStackTrace();
                        }
                    }
                }
            }
        });
    }

    @ReactMethod
    public void checkUpgrade() {
        OkHttpClient okHttpClient = new OkHttpClient();
        String upgradeUrl = HOST.concat("versionCheck");
        PackageManager manager = reactContext.getPackageManager();
        try {
            String packageName = reactContext.getPackageName();
            PackageInfo info = manager.getPackageInfo(packageName, 0);
            String versionName = info.versionName;
            upgradeUrl = upgradeUrl
                    .concat("?packageName=")
                    .concat(packageName)
                    .concat("&version=")
                    .concat(versionName);
        } catch (PackageManager.NameNotFoundException e) {
            Log.e("Version Upgrade", "NOT FOUND MY PACKAGE TAT", e);
            return;
        }
        final Request request = new Request.Builder()
                .url(upgradeUrl)
                .method("GET", null)
                .build();
        Call call = okHttpClient.newCall(request);
        final Handler handler = new ShowUpgradeAlertHandler(this);
        call.enqueue(new Callback() {
            @Override
            public void onFailure(Call call, IOException e) {
                Log.e("VERSION CHECK", call.toString(), e);
            }

            @Override
            public void onResponse(Call call, Response response) {
                if (response.isSuccessful()) {
                    if (response.body() == null) return;
                    try {
                        String jsonText = response.body().string();
                        Log.d("VERSION CHECK", jsonText);
                        JSONObject json = new JSONObject(jsonText);
                        int status = json.getInt("status");
                        if (status > 0) {
                            CONTENT = json.getString("message");
//                            String version = json.getString("version");
                            JSONObject jsonData = json.getJSONObject("data");
                            DOWNLOAD_URL = jsonData.getString("url");
                            handler.sendEmptyMessage(0);
                        }
                    } catch (IOException e) {
                        Log.e("REQUEST FAILED", "AAA", e);
                    } catch (JSONException e) {
                        Log.e("JSON FAILED", "BBB", e);
                    }
                }
            }
        });
    }
}

final class ShowUpgradeAlertHandler extends Handler {
    private UpgradeModule module;

    ShowUpgradeAlertHandler(UpgradeModule module) {
        this.module = module;
    }

    @Override
    public void handleMessage(Message message) {
        module._alertDialog();
    }
}

final class DoInstallHandler extends Handler {
    private UpgradeModule module;

    DoInstallHandler(UpgradeModule module) {
        this.module = module;
    }

    @Override
    public void handleMessage(Message message) {
        if(module._hasPermissionToInstall()) {
            if(message.what == 0){
                String fileUri = (String)message.obj;
                module.installPackage(fileUri);
            }
        }
    }
}